/**
 * @author Ernesto Casablanca (casablancaernesto@gmail.com)
 * @copyright 2024 dlinear
 * @licence BSD 3-Clause License
 * Scanner utility class.
 *
 * This class extends from the VnnlibFlexLexer class generated by Flex. It
 * provides a class named VnnlibScanner used to parse the file using the Flex
 * API.
 */
#pragma once

#ifndef __DLINEAR_VNNLIB_SCANNER_H__
#define yyFlexLexer VnnlibFlexLexer
#include <FlexLexer.h>
#undef yyFlexLexer
#endif

#ifdef DLINEAR_PYDLINEAR
#include "pydlinear/interrupt.h"
#endif

#include "dlinear/parser/vnnlib/Sort.h"
#include "dlinear/parser/vnnlib/Term.h"
#include "dlinear/symbolic/symbolic.h"
#include "dlinear/util/Box.h"
// The following include should come after all the previous ones.
// Do not alpha-sort them.
#include "dlinear/parser/vnnlib/parser.yy.hpp"

namespace dlinear::vnnlib {

/**
 * VnnlibScanner is a derived class to add some extra function to the scanner
 * class. Flex itself creates a class named yyFlexLexer, which is renamed using
 * macros to ExampleFlexLexer. However we change the context of the generated
 * yylex() function to be contained within the VnnlibScanner class. This is
 * required because the yylex() defined in ExampleFlexLexer has no parameters.
 */
class VnnlibScanner : public VnnlibFlexLexer {
 public:
  /**
   * Create a new scanner object. The streams arg_yyin and arg_yyout default
   * to cin and cout, but that assignment is only made when initializing in
   * yylex().
   */
  explicit VnnlibScanner(std::istream *arg_yyin = nullptr, std::ostream *arg_yyout = nullptr);

  VnnlibScanner(const VnnlibScanner &) = delete;
  VnnlibScanner(VnnlibScanner &&) = delete;
  VnnlibScanner &operator=(const VnnlibScanner &) = delete;
  VnnlibScanner &operator=(VnnlibScanner &&) = delete;

  /** Required for virtual functions */
  ~VnnlibScanner() override;

  /**
   * This is the main lexing function. It is generated by flex according to
   * the macro declaration YY_DECL above. The generated bison parser then
   * calls this virtual function to fetch new tokens.
   */
  virtual VnnlibParser::token_type lex(VnnlibParser::semantic_type *yylval, VnnlibParser::location_type *yylloc);

  /** Enable debug output (via arg_yyout) if compiled into the scanner. */
  void set_debug(bool b);
};

}  // namespace dlinear::vnnlib
